@@ -3,11 +3,12 @@ use crate::formatters::trivia_formatter::{self, FormatTriviaType};
 use full_moon::ast::types::{AsAssertion, IndexedTypeInfo, TypeField, TypeFieldKey, TypeInfo};
 use full_moon::{
     ast::{
-        span::ContainedSpan, BinOp, Call, Expression, Field, FunctionArgs, Index, Prefix, Suffix,
-        TableConstructor, UnOp, Value, Var,
+        punctuated::Pair, span::ContainedSpan, BinOp, Call, Expression, Field, FunctionArgs, Index,
+        Prefix, Stmt, Suffix, TableConstructor, UnOp, Value, Var,
     },
     tokenizer::{Token, TokenKind, TokenReference, TokenType},
 };
+use std::borrow::Cow;
 
 pub fn trivia_contains_newline<'ast>(trivia_vec: impl Iterator<Item = &'ast Token<'ast>>) -> bool {
     for trivia in trivia_vec {
@@ -297,6 +298,109 @@ pub fn get_expression_trailing_comments<'ast>(
     (new_expression, trailing_comments)
 }
 
+pub fn get_stmt_trailing_trivia<'ast>(stmt: Stmt<'ast>) -> (Stmt<'ast>, Vec<Token<'ast>>) {
+    let mut trailing_trivia = Vec::new();
+    let updated_stmt = match stmt {
+        Stmt::Assignment(assignment) => {
+            let mut formatted_expression_list = assignment.expr_list().to_owned();
+            if let Some(last_pair) = formatted_expression_list.pop() {
+                match last_pair {
+                    Pair::End(value) => {
+                        trailing_trivia = get_expression_trailing_trivia(&value);
+                        let expression = trivia_formatter::expression_add_trailing_trivia(
+                            value,
+                            FormatTriviaType::Replace(vec![]),
+                        );
+                        formatted_expression_list.push(Pair::End(expression));
+                    }
+                    Pair::Punctuated(_, _) => {
+                        panic!("we got a punctuated as the last sequence in expression")
+                    }
+                }
+            }
+
+            Stmt::Assignment(assignment.with_expr_list(formatted_expression_list))
+        }
+
+        Stmt::LocalAssignment(local_assignment) => {
+            let new_assignment = if local_assignment.expr_list().is_empty() {
+                // Unassigned local variable
+                let mut formatted_name_list = local_assignment.name_list().to_owned();
+
+                // Retrieve last item and take its trailing comments
+                if let Some(last_pair) = formatted_name_list.pop() {
+                    match last_pair {
+                        Pair::End(value) => {
+                            trailing_trivia =
+                                value.trailing_trivia().map(|x| x.to_owned()).collect();
+                            let value = Cow::Owned(trivia_formatter::token_reference_add_trivia(
+                                value.into_owned(),
+                                FormatTriviaType::NoChange,
+                                FormatTriviaType::Replace(vec![]),
+                            ));
+                            formatted_name_list.push(Pair::End(value));
+                        }
+                        Pair::Punctuated(_, _) => {
+                            panic!("punctuated sequence not ended with a Pair::End")
+                        }
+                    }
+                }
+                local_assignment.with_name_list(formatted_name_list)
+            } else {
+                // Add newline at the end of LocalAssignment expression list
+                // Expression list should already be formatted
+                let mut formatted_expression_list = local_assignment.expr_list().to_owned();
+
+                // Retrieve last item and remove trailing trivia
+                if let Some(last_pair) = formatted_expression_list.pop() {
+                    match last_pair {
+                        Pair::End(value) => {
+                            trailing_trivia = get_expression_trailing_trivia(&value);
+                            let expression = trivia_formatter::expression_add_trailing_trivia(
+                                value,
+                                FormatTriviaType::Replace(vec![]),
+                            );
+                            formatted_expression_list.push(Pair::End(expression));
+                        }
+                        Pair::Punctuated(_, _) => {
+                            panic!("got a last pair which was punctuated")
+                        }
+                    }
+                }
+
+                local_assignment.with_expr_list(formatted_expression_list)
+            };
+
+            Stmt::LocalAssignment(new_assignment)
+        }
+
+        Stmt::FunctionCall(function_call) => {
+            let last_suffix = function_call.iter_suffixes().last();
+            trailing_trivia = match last_suffix {
+                Some(suffix) => suffix_trailing_trivia(suffix),
+                None => Vec::new(),
+            };
+
+            Stmt::FunctionCall(trivia_formatter::function_call_add_trailing_trivia(
+                function_call,
+                FormatTriviaType::Replace(vec![]),
+            ))
+        }
+        Stmt::Repeat(repeat_block) => {
+            trailing_trivia = get_expression_trailing_trivia(repeat_block.until());
+            let until_expr = trivia_formatter::expression_add_trailing_trivia(
+                repeat_block.until().to_owned(),
+                FormatTriviaType::Replace(vec![]),
+            );
+
+            Stmt::Repeat(repeat_block.with_until(until_expr))
+        }
+        _ => panic!("stmt trailing comments not implemented"),
+    };
+
+    (updated_stmt, trailing_trivia)
+}
+
 pub fn token_trivia_contains_comments<'ast>(
     trivia: impl Iterator<Item = &'ast Token<'ast>>,
 ) -> bool {
@@ -562,6 +666,67 @@ fn as_assertion_contains_comments<'ast>(as_assertion: &AsAssertion<'ast>) -> boo
         || type_info_contains_comments(as_assertion.cast_to())
 }
 
+fn value_contains_comments<'ast>(value: &Value<'ast>) -> bool {
+    match value {
+        Value::Function((token, body)) => {
+            if token_contains_comments(token) {
+                true
+            } else {
+                contained_span_contains_comments(body.parameters_parentheses())
+                // TODO: Do we need to do any more?
+            }
+        }
+        Value::FunctionCall(function_call) => {
+            let contained = match function_call.prefix() {
+                Prefix::Name(token) => token_contains_comments(token),
+                Prefix::Expression(expression) => expression_contains_comments(expression),
+            };
+
+            if contained {
+                true
+            } else {
+                let mut contained_comments = false;
+                for suffix in function_call.iter_suffixes() {
+                    contained_comments = suffix_contains_comments(suffix);
+                    if contained_comments {
+                        break;
+                    }
+                }
+                contained_comments
+            }
+        }
+        Value::TableConstructor(table_constructor) => {
+            table_constructor_contains_comments(table_constructor)
+        }
+        Value::Number(token) => token_contains_comments(token),
+        Value::ParseExpression(expression) => expression_contains_comments(expression),
+        Value::String(token) => token_contains_comments(token),
+        Value::Symbol(token) => token_contains_comments(token),
+        Value::Var(var) => match var {
+            Var::Name(token) => token_contains_comments(token),
+            Var::Expression(var_expr) => {
+                let contained = match var_expr.prefix() {
+                    Prefix::Name(token) => token_contains_comments(token),
+                    Prefix::Expression(expression) => expression_contains_comments(expression),
+                };
+
+                if contained {
+                    true
+                } else {
+                    let mut contained_comments = false;
+                    for suffix in var_expr.iter_suffixes() {
+                        contained_comments = suffix_contains_comments(suffix);
+                        if contained_comments {
+                            break;
+                        }
+                    }
+                    contained_comments
+                }
+            }
+        },
+    }
+}
+
 // Check whether any comments are present within an Expression
 pub fn expression_contains_comments<'ast>(expression: &Expression<'ast>) -> bool {
     match expression {
@@ -588,68 +753,7 @@ pub fn expression_contains_comments<'ast>(expression: &Expression<'ast>) -> bool
             #[cfg(feature = "luau")]
             as_assertion,
         } => {
-            let contains_comments = match &**value {
-                Value::Function((token, body)) => {
-                    if token_contains_comments(token) {
-                        true
-                    } else {
-                        contained_span_contains_comments(body.parameters_parentheses())
-                        // TODO: Do we need to do any more?
-                    }
-                }
-                Value::FunctionCall(function_call) => {
-                    let contained = match function_call.prefix() {
-                        Prefix::Name(token) => token_contains_comments(token),
-                        Prefix::Expression(expression) => expression_contains_comments(expression),
-                    };
-
-                    if contained {
-                        true
-                    } else {
-                        let mut contained_comments = false;
-                        for suffix in function_call.iter_suffixes() {
-                            contained_comments = suffix_contains_comments(suffix);
-                            if contained_comments {
-                                break;
-                            }
-                        }
-                        contained_comments
-                    }
-                }
-                Value::TableConstructor(table_constructor) => {
-                    table_constructor_contains_comments(table_constructor)
-                }
-                Value::Number(token) => token_contains_comments(token),
-                Value::ParseExpression(expression) => expression_contains_comments(expression),
-                Value::String(token) => token_contains_comments(token),
-                Value::Symbol(token) => token_contains_comments(token),
-                Value::Var(var) => match var {
-                    Var::Name(token) => token_contains_comments(token),
-                    Var::Expression(var_expr) => {
-                        let contained = match var_expr.prefix() {
-                            Prefix::Name(token) => token_contains_comments(token),
-                            Prefix::Expression(expression) => {
-                                expression_contains_comments(expression)
-                            }
-                        };
-
-                        if contained {
-                            true
-                        } else {
-                            let mut contained_comments = false;
-                            for suffix in var_expr.iter_suffixes() {
-                                contained_comments = suffix_contains_comments(suffix);
-                                if contained_comments {
-                                    break;
-                                }
-                            }
-                            contained_comments
-                        }
-                    }
-                },
-            };
-
-            if contains_comments {
+            if value_contains_comments(value) {
                 true
             } else {
                 let binop_contains_comments = match binop {
@@ -694,13 +798,48 @@ pub fn expression_contains_comments<'ast>(expression: &Expression<'ast>) -> bool
 
 // Checks to see whether an expression contains comments inline inside of it
 // This can only happen if the expression is a BinOp
+// We should ignore any comments which are trailing for the whole expression, as they are not inline
 pub fn expression_contains_inline_comments<'ast>(expression: &Expression<'ast>) -> bool {
     match expression {
-        Expression::Value { binop, .. } => {
-            if binop.is_some() {
-                expression_contains_comments(expression)
-            } else {
-                false
+        Expression::Value { binop, value, .. } => {
+            match binop {
+                Some(binop_rhs) => {
+                    let rhs = binop_rhs.rhs();
+                    let contains = match binop_rhs.bin_op() {
+                        BinOp::And(t)
+                        | BinOp::Caret(t)
+                        | BinOp::GreaterThan(t)
+                        | BinOp::GreaterThanEqual(t)
+                        | BinOp::LessThan(t)
+                        | BinOp::LessThanEqual(t)
+                        | BinOp::Minus(t)
+                        | BinOp::Or(t)
+                        | BinOp::Percent(t)
+                        | BinOp::Plus(t)
+                        | BinOp::Slash(t)
+                        | BinOp::Star(t)
+                        | BinOp::TildeEqual(t)
+                        | BinOp::TwoDots(t)
+                        | BinOp::TwoEqual(t) => token_contains_comments(t),
+                    };
+                    contains
+                        || value_contains_comments(value)
+                        // Check if the binop chain still continues
+                        // If so, we should keep checking the expresion
+                        // Otherwise, stop checking
+                        || match rhs {
+                            Expression::Value { binop, value, .. } => {
+                                if binop.is_some() {
+                                    value_contains_comments(value)
+                                        || expression_contains_inline_comments(rhs)
+                                } else {
+                                    false
+                                }
+                            }
+                            _ => expression_contains_comments(rhs),
+                        }
+                }
+                None => false,
             }
         }
         _ => false,
